package tempest.app.manager;

import org.eclipse.jface.dialogs.IPageChangedListener;
import org.eclipse.jface.dialogs.PageChangedEvent;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TaskBar;
import org.eclipse.swt.widgets.TaskItem;

import tempest.app.manager.composite.AchievementComposite;
import tempest.app.manager.composite.GestureComposite;
import tempest.app.manager.composite.HomeComposite;
import tempest.app.manager.composite.PropertiesComposite;
import tempest.app.manager.dialogs.GenerateThumbnails;
import tempest.app.manager.utils.Configuration;
import tempest.app.manager.utils.GameConfiguration;
import tempest.app.manager.wizard.ManagerWizard;
import tempest.app.manager.wizard.ThirdFileReview;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.wb.swt.SWTResourceManager;

public class GameManager {
	public Shell shell;
	public String filename;
	protected StackLayout layout;

	public static final String APP_NAME = "Tempest Game Manager";

	public static PropertiesComposite properties;
	public static HomeComposite home;
	public static GestureComposite gestures;
	public static AchievementComposite achievements;

	private MenuItem mntmGestures;
	private Menu menu_2;
	private MenuItem mntmAchievements;
	private Menu menu_3;
	private MenuItem mntmProperties;
	private Menu menu_4;
	private MenuItem mntmHelp;
	private Menu menu_5;
	private MenuItem mntmAbout;

	public enum Layouts {
		HOME, PROPERTIES, GESTURES, ACHIEVEMENTS;
	}

	public TaskItem getTaskBarItem () {
		TaskBar bar = shell.getDisplay().getSystemTaskBar();
		if (bar == null) return null;
		TaskItem item = bar.getItem(shell);
		if (item == null) item = bar.getItem(null);
		return item;
	}

	public static void main(String[] args) {
		try {
			GameManager window = new GameManager();
			window.open();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void open() {
		Configuration.loadInstance();
		Display display = Display.getDefault();

		createContents();

		if (Configuration.asString("Project.name", "") == "") {
			final WizardDialog wizard = new WizardDialog(shell, new ManagerWizard());
			wizard.addPageChangedListener(new IPageChangedListener() {

				@Override
				public void pageChanged(PageChangedEvent event) {
					if (wizard.getCurrentPage() instanceof ThirdFileReview) {
						ThirdFileReview page = (ThirdFileReview) wizard.getCurrentPage();
						page.updateFields();
						page.setPageComplete(true);
					}
				}
			});

			if (wizard.open() != Window.OK) {
				System.out.println("Cancel pressed");
				shell.dispose();
				display.dispose();
			}
		}
		initialization();

		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}
	}

	private void initialization() {
		GameConfiguration.loadInstance();
		properties.updateStats();
		home.seedAchievementsList();
		home.seedGesturesList();

		shell.open();
		shell.layout();
	}

	public void switchTo(Layouts composite) {
		switch (composite) {
		case HOME:
			layout.topControl = home;
			shell.layout();
			break;
		case PROPERTIES:
			layout.topControl = properties;
			shell.layout();
			break;
		case GESTURES:
			layout.topControl = gestures;
			gestures.updateComposite();
			shell.layout();
			break;
		case ACHIEVEMENTS:
			layout.topControl = achievements;
			//achievements.openAchievements(Configuration.asString("Achievements.xml", null));
			shell.layout();
			break;
		}
	}

	protected void createContents() {
		shell = new Shell();
		shell.setText(APP_NAME + " - " + Configuration.asString("Project.name", ""));

		compositeMenus();

		home = new HomeComposite(shell, SWT.NONE, this);
		gestures = new GestureComposite(shell, SWT.NONE, this);
		properties = new PropertiesComposite(shell, SWT.NONE);
		achievements = new AchievementComposite(shell, SWT.NONE, this);

		layout = new StackLayout();
		shell.setLayout(layout);

		layout.topControl = home;
	}

	private void compositeMenus() {
		Menu menu = new Menu(shell, SWT.BAR);
		shell.setMenuBar(menu);

		MenuItem mntmFile = new MenuItem(menu, SWT.CASCADE);
		mntmFile.setText("&File");

		Menu menu_1 = new Menu(mntmFile);
		mntmFile.setMenu(menu_1);

		MenuItem mntmCreateNew = new MenuItem(menu_1, SWT.CASCADE);
		mntmCreateNew.setImage(SWTResourceManager.getImage(GameManager.class, "/tempest/app/assets/new.png"));
		mntmCreateNew.setText("Create &new...");

		Menu menu_6 = new Menu(mntmCreateNew);
		mntmCreateNew.setMenu(menu_6);

		MenuItem mntmAchievementXml = new MenuItem(menu_6, SWT.NONE);
		mntmAchievementXml.setImage(SWTResourceManager.getImage(GameManager.class, "/tempest/app/assets/xml.png"));
		mntmAchievementXml.setText("Achievement &XML");

		MenuItem mntmGestureLibrary = new MenuItem(menu_6, SWT.NONE);
		mntmGestureLibrary.setImage(SWTResourceManager.getImage(GameManager.class, "/tempest/app/assets/gestures.png"));
		mntmGestureLibrary.setText("&Gesture library");

		MenuItem mntmPropertiesFile = new MenuItem(menu_6, SWT.NONE);
		mntmPropertiesFile.setImage(SWTResourceManager.getImage(GameManager.class, "/tempest/app/assets/properties.png"));
		mntmPropertiesFile.setText("&Properties file");

		new MenuItem(menu_1, SWT.SEPARATOR);

		MenuItem mntmOpen = new MenuItem(menu_1, SWT.NONE);
		mntmOpen.setImage(SWTResourceManager.getImage(GameManager.class, "/tempest/app/assets/open.png"));
		mntmOpen.setText("&Open...\t Ctrl+O");
		mntmOpen.setAccelerator(SWT.CTRL + 'O');

		new MenuItem(menu_1, SWT.SEPARATOR);

		MenuItem mntmSave = new MenuItem(menu_1, SWT.NONE);
		mntmSave.setImage(SWTResourceManager.getImage(GameManager.class, "/tempest/app/assets/save.png"));
		mntmSave.setText("&Save\t Ctrl+S");
		mntmSave.setAccelerator(SWT.CTRL + 'S');

		MenuItem mntmSaveAs = new MenuItem(menu_1, SWT.NONE);
		mntmSaveAs.setImage(SWTResourceManager.getImage(GameManager.class, "/tempest/app/assets/save-as.png"));
		mntmSaveAs.setText("Save &As...\t Ctrl+Shirf+S");
		mntmSaveAs.setAccelerator(SWT.CTRL + SWT.SHIFT + 'S');

		MenuItem mntmSaveAll = new MenuItem(menu_1, SWT.NONE);
		mntmSaveAll.setText("Save All");

		new MenuItem(menu_1, SWT.SEPARATOR);

		MenuItem mntmWelcomeScreen = new MenuItem(menu_1, SWT.NONE);
		mntmWelcomeScreen.setImage(SWTResourceManager.getImage(GameManager.class, "/tempest/app/assets/home.png"));
		mntmWelcomeScreen.setText("Welcome screen\t &Home");
		mntmWelcomeScreen.setAccelerator(SWT.HOME);

		new MenuItem(menu_1, SWT.SEPARATOR);

		MenuItem mntmExit = new MenuItem(menu_1, SWT.NONE);
		mntmExit.setImage(SWTResourceManager.getImage(GameManager.class, "/tempest/app/assets/exit.png"));
		mntmExit.setText("E&xit\t Alt+F4");

		mntmGestures = new MenuItem(menu, SWT.CASCADE);
		mntmGestures.setText("&Gestures");

		menu_2 = new Menu(mntmGestures);
		mntmGestures.setMenu(menu_2);

		new MenuItem(menu_2, SWT.SEPARATOR);

		MenuItem mntmAddGesture = new MenuItem(menu_2, SWT.NONE);
		mntmAddGesture.setImage(SWTResourceManager.getImage(GameManager.class, "/tempest/app/assets/add.png"));
		mntmAddGesture.setText("Add &gesture\t Ins");
		mntmAddGesture.setAccelerator(SWT.INSERT);

		MenuItem mntmRemoveGesture = new MenuItem(menu_2, SWT.NONE);
		mntmRemoveGesture.setImage(SWTResourceManager.getImage(GameManager.class, "/tempest/app/assets/remove.png"));
		mntmRemoveGesture.setText("&Remove gesture\t Del");
		mntmRemoveGesture.setAccelerator(SWT.DEL);

		new MenuItem(menu_2, SWT.SEPARATOR);

		MenuItem mntmGenerateThumbnails = new MenuItem(menu_2, SWT.NONE);
		mntmGenerateThumbnails.setImage(SWTResourceManager.getImage(GameManager.class, "/tempest/app/assets/image.png"));
		mntmGenerateThumbnails.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				GenerateThumbnails generate = new GenerateThumbnails(GameManager.this);
				generate.open();
			}
		});
		mntmGenerateThumbnails.setText("Generate &thumbnails\t Ctrl+T");
		mntmGenerateThumbnails.setAccelerator(SWT.CTRL + 'T');

		mntmAchievements = new MenuItem(menu, SWT.CASCADE);
		mntmAchievements.setText("&Achievements");

		menu_3 = new Menu(mntmAchievements);
		mntmAchievements.setMenu(menu_3);

		new MenuItem(menu_3, SWT.SEPARATOR);

		MenuItem mntmNewItem = new MenuItem(menu_3, SWT.NONE);
		mntmNewItem.setImage(SWTResourceManager.getImage(GameManager.class, "/tempest/app/assets/validate.png"));
		mntmNewItem.setText("Validate now");

		MenuItem mntmNewCheckbox = new MenuItem(menu_3, SWT.CHECK);
		mntmNewCheckbox.setSelection(true);
		mntmNewCheckbox.setText("Auto validate XML");

		mntmProperties = new MenuItem(menu, SWT.CASCADE);
		mntmProperties.setText("&Properties");

		menu_4 = new Menu(mntmProperties);
		mntmProperties.setMenu(menu_4);

		mntmHelp = new MenuItem(menu, SWT.CASCADE);
		mntmHelp.setText("&Help");

		menu_5 = new Menu(mntmHelp);
		mntmHelp.setMenu(menu_5);

		MenuItem mntmGetHelp = new MenuItem(menu_5, SWT.NONE);
		mntmGetHelp.setImage(SWTResourceManager.getImage(GameManager.class, "/tempest/app/assets/help.png"));
		mntmGetHelp.setText("Get &help");

		mntmAbout = new MenuItem(menu_5, SWT.NONE);
		mntmAbout.setImage(SWTResourceManager.getImage(GameManager.class, "/tempest/app/assets/info.png"));
		mntmAbout.setText("&About");
	}
}
