package tempest.app.manager.composite;

import java.io.File;
import java.io.IOException;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.SWT;

import tempest.app.manager.achievements.AchievementValidator;
import tempest.app.manager.achievements.AchievementParser;
import tempest.app.manager.factories.GoalFactory;
import tempest.app.manager.GameManager;

import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.wb.swt.SWTResourceManager;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.TreeItem;

public class AchievementComposite extends Composite {
	private Text textID;
	private Text textType;

	private GameManager owner;
	private GoalFactory factory;

	public AchievementParser parser;
	public AchievementValidator validator = new AchievementValidator();

	public static Text validateText;
	private Text text;
	public Tree tree;
	private Group goalGroup;
	private Canvas canvas;
	private Combo combo;

	public AchievementComposite(Composite parent, int style, final GameManager owner) {
		super(parent, style);
		this.owner = owner;

		parser = new AchievementParser();
		factory = new GoalFactory();

		configLayout();

		SashForm sashForm = new SashForm(this, SWT.NONE);

		tree = new Tree(sashForm, SWT.BORDER);
		tree.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				final TreeItem current = (TreeItem) e.item;
				for (TreeItem item : tree.getItems()) {
					if(current.equals(item)) {
						for (Widget w : goalGroup.getChildren()) {
							w.dispose();
						}

						parser.setCurrent(tree.indexOf(tree.getSelection()[0]));

						textID.setText(current.getItems()[0].getText().substring(4));
						textType.setText(current.getText());

						for (TreeItem goal : current.getItems()[2].getItems()) {
							factory.addGoalToGroup(goalGroup, goal.getText().split(":")[0], goal.getText().split(":")[1]);
						}
						factory.addEditFields(goalGroup);
						goalGroup.layout();
						canvas.redraw();
						canvas.addPaintListener(new PaintListener() {

							private Image image;

							@Override
							public void paintControl(PaintEvent e) {
								e.gc.setAntialias(SWT.ON);
								e.gc.setInterpolation(SWT.HIGH);

								e.gc.setForeground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
								e.gc.fillRectangle(getBounds());
								if (new File(new File(owner.filename).getParent() + "\\" + current.getItem(1).getText().split(": ")[1]).exists()) {
									image = new Image(getDisplay(), new File(owner.filename).getParent() + "\\" + current.getItem(1).getText().split(": ")[1]);
								} else {
									image = (SWTResourceManager.getImage(GameManager.class, "/tempest/app/assets/icon.png"));
								}

								e.gc.drawImage(image, 0, 0, image.getBounds().width, image.getBounds().height, 
										0, 0, canvas.getSize().x, canvas.getSize().y);

							}
						});
					}
				}
			}
		});


		Composite composite = new Composite(sashForm, SWT.NONE);
		composite.setLayout(new FillLayout(SWT.VERTICAL));

		Group grpAchievement = new Group(composite, SWT.NONE);
		grpAchievement.setText("Achievement");
		grpAchievement.setLayout(new GridLayout(2, false));

		canvas = new Canvas(grpAchievement, SWT.BORDER);
		canvas.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
		canvas.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));

		Composite composite_1 = new Composite(grpAchievement, SWT.NONE);
		GridData gd_composite_1 = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
		gd_composite_1.heightHint = 54;
		gd_composite_1.widthHint = 118;
		composite_1.setLayoutData(gd_composite_1);
		composite_1.setLayout(new GridLayout(2, false));

		Label lblType = new Label(composite_1, SWT.NONE);
		lblType.setText("Type:");

		textType = new Text(composite_1, SWT.BORDER);
		textType.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

		Label lblId = new Label(composite_1, SWT.NONE);
		lblId.setText("ID:");

		textID = new Text(composite_1, SWT.BORDER);
		textID.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

		goalGroup = new Group(composite, SWT.NONE);
		goalGroup.setText("Goals");
		goalGroup.setLayout(new GridLayout(3, false));

		factory.addEditFields(goalGroup);

		Group grpValidate = new Group(composite, SWT.NONE);
		grpValidate.setText("Validate");
		grpValidate.setLayout(new FillLayout(SWT.HORIZONTAL));

		validateText = new Text(grpValidate, SWT.BORDER | SWT.READ_ONLY | SWT.WRAP | SWT.MULTI);
		sashForm.setWeights(new int[] {1, 2});
	}

	public void openAchievements(String filename) {
		String input;
		//		if(owner.filename != null)
		//			savePrompt();
		if (filename == null) {
			FileDialog open = new FileDialog(owner.shell, SWT.OPEN);
			open.setFilterExtensions(new String[]{"*.xml", "*.*"});
			open.setFilterNames(new String[]{"Achievements XML(*.xml)", "All files(.*.)"});
			open.setFilterIndex(0);
			open.setText("Open achievements xml");

			input = open.open();
		} else {
			input = filename;
		}

		if (input == null)
			input = owner.filename;
		owner.filename = input;

		parser.setFilename(owner.filename);
		try {
			parser.populate(tree);
			validator.validate(owner.filename);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void configLayout() {
		FillLayout fillLayout = new FillLayout(SWT.HORIZONTAL);
		fillLayout.spacing = 5;
		fillLayout.marginHeight = 5;
		fillLayout.marginWidth = 5;
		setLayout(fillLayout);
	}

	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}
}
