package tempest.app.manager.composite;

import java.util.ArrayList;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.ImageLoader;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.wb.swt.SWTResourceManager;

import tempest.game.arena.core.gestures.Gesture;
import tempest.game.arena.core.gestures.GestureConstants;
import tempest.game.arena.core.gestures.GestureLibrary;
import tempest.game.arena.core.gestures.GestureStroke;
import tempest.game.arena.core.gestures.Prediction;

import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.utils.Array;

public class GestureCanvas extends Canvas {
	private float[] gesture;
	private Rectangle bounds;
	private ArrayList<Vector2> strokes;
	private GestureLibrary mLibrary;
	private boolean gesturing;

	private boolean touchDown = false;

	private int mX;
	private int mY;

	private int mTotalLength = 0;
	private float mGestureStrokeLengthThreshold = 100.0f;

	private boolean mIsGesturing;

	private Gesture mCurrentGesture;

	public static Array<Vector2> mPointBuffer = new Array<Vector2>(
			GestureConstants.STROKE_POINT_BUFFER_SIZE);

	public GestureCanvas(final Composite parent, int style) {
		super(parent, style);

		gesturing = false;
		gesture = new float[1024];
		strokes = new ArrayList<Vector2>();

		this.addPaintListener(new PaintListener() {

			@Override
			public void paintControl(PaintEvent e) {
				configGC(e.gc);

				if(gesturing) {
					getInputGesture(e);
				} else {
					if(mLibrary != null) {
						drawGesture(e);
					}
				}
			}

			private void drawGesture(final PaintEvent e) {
				try {
					centerGesture();
					e.gc.setLineWidth(5);

					for (int i = 0; i < gesture.length; i+=2) {
						e.gc.drawLine((int) gesture[i], (int) gesture[i+1], (int) gesture[i+2],(int) gesture[i+3]);
					}
				} catch (IndexOutOfBoundsException ex) {

				} finally {
					e.gc.setForeground(SWTResourceManager.getColor(SWT.COLOR_GREEN));
					e.gc.drawLine((int) gesture[0], (int) gesture[1], (int)gesture[2], (int)gesture[3]);


					e.gc.setForeground(SWTResourceManager.getColor(SWT.COLOR_RED));
					e.gc.drawLine((int) gesture[gesture.length-4], (int) gesture[gesture.length-3], 
							(int) gesture[gesture.length-2], (int) gesture[gesture.length-1]);

					e.gc.setForeground(SWTResourceManager.getColor(SWT.COLOR_DARK_GRAY));
					e.gc.setLineWidth(1);

					try {
						e.gc.drawRectangle((int) bounds.x, (int) bounds.y, (int) bounds.width, (int) bounds.height);
					} catch (NullPointerException npe) {
						// TODO: handle exception
					}
				}
			}

			private void centerGesture() {;
			try {
				Vector2 centerPosition =  new Vector2();

				centerPosition.x = ((getSize().x / 2) - (bounds.width / 2)) - bounds.x;
				centerPosition.y = ((getSize().y / 2) - (bounds.height / 2)) - bounds.y;

				bounds.x += centerPosition.x;
				bounds.y += centerPosition.y;

				for (int i = 0; i < gesture.length; i+=2) {
					gesture[i] += centerPosition.x;
					gesture[i+1] += centerPosition.y;
				}
			} catch (NullPointerException npe) {
				// TODO: handle exception
			}
			}

			private void getInputGesture(final PaintEvent e) {
				try {
					for (int i = 0; i < strokes.size(); i++) {
						e.gc.drawLine((int) strokes.get(i).x, (int) strokes.get(i).y, 
								(int) strokes.get(i+1).x, (int) strokes.get(i+1).y);
					}
				} catch(IndexOutOfBoundsException ex) {
					//ex.printStackTrace();
				}

			}
		});

		this.addMouseListener(new MouseListener() {

			@Override
			public void mouseUp(MouseEvent e) {
				touchDown = false;
				if (mCurrentGesture != null && (mPointBuffer.size != 0)) {

					mCurrentGesture.addStroke(new GestureStroke(mPointBuffer));

					if(mLibrary != null)
						recognizeGesture();

					mIsGesturing = false;
				}
			}

			@Override
			public void mouseDown(MouseEvent e) {
				mCurrentGesture = null;
				mPointBuffer.clear();
				touchDown = true;
				resetStroke();

				mX = e.x;
				mY = e.y;

				mTotalLength = 0;
				mIsGesturing = false;

				if (mCurrentGesture == null) {
					mCurrentGesture = new Gesture();
				}

				mPointBuffer.add(new Vector2(e.x, e.y));
				addStroke(new Vector2(e.x, e.y));
				redraw();
			}

			@Override
			public void mouseDoubleClick(MouseEvent e) {}

			private void recognizeGesture() {
				if (mTotalLength > mGestureStrokeLengthThreshold) {
					Array<Prediction> predictions = mLibrary.recognize(mCurrentGesture);

					if (predictions.size > 0) {
						Prediction prediction = predictions.get(0);

						if (prediction.score > 1.0) {
							GestureComposite.gestureName.setText("Recognized as " + prediction.name);
							System.out.println("Gesture is: " + prediction.name);
						} else {
							GestureComposite.gestureName.setText("Gesture not recognized...");
						}
					}
				}
			}
		});

		this.addMouseMoveListener(new MouseMoveListener() {

			@Override
			public void mouseMove(MouseEvent e) {
				if(touchDown ) {

					final int previousX = mX;
					final int previousY = mY;

					final int dx = Math.abs(e.x - previousX);
					final int dy = Math.abs(e.y - previousY);

					if (dx >= GestureStroke.TOUCH_TOLERANCE || dy >=  GestureStroke.TOUCH_TOLERANCE) {
						mX = e.x;
						mY = e.y;

						Vector2 vector2 = new Vector2(e.x, e.y);
						mPointBuffer.add(vector2);
						addStroke(vector2);
						redraw();

						if (!mIsGesturing) {
							mTotalLength += (float) Math.sqrt(dx * dx + dy * dy);

							if (mTotalLength > mGestureStrokeLengthThreshold) {
								System.out.println(mTotalLength + " " + mGestureStrokeLengthThreshold);
								mIsGesturing = true;
							}
						}
					}
				}
			}
		});
	}

	public void generateThumbs(String folder, int width, int height) {
		if(mLibrary != null) {
			for (String s : mLibrary.getGestureEntries()) {
				Gesture g = mLibrary.getGestures(s).get(0);
				bounds = g.getStrokes().get(0).boundingBox;
				gesture = g.getStrokes().get(0).points;
				generateImage(folder, s, width, height);
				System.out.println("Generated: " + s);
			}
		}
	}

	private void generateImage(String folder, String name, int width, int height) {
		int size = (int) (bounds.width > bounds.height ? bounds.width : bounds.height);
		Image image = new Image(getDisplay(), size, size);
		GC gc = new GC(image);
		gc.setAdvanced(true);
		gc.setAntialias(SWT.ON);
		gc.setInterpolation(SWT.HIGH);

		try {
			centerGesture(size);

			if (width < 32) {
				gc.setLineWidth(20);
			} else if (width > 32) {
				gc.setLineWidth(10);
			}

			for (int i = 0; i < gesture.length; i+=2) {
				gc.drawLine((int) gesture[i], (int) gesture[i+1], (int) gesture[i+2],(int) gesture[i+3]);
			}
		} catch (IndexOutOfBoundsException ex) {

		} finally {
			gc.setForeground(SWTResourceManager.getColor(SWT.COLOR_GREEN));
			gc.drawLine((int) gesture[0], (int) gesture[1], (int)gesture[2], (int)gesture[3]);


			gc.setForeground(SWTResourceManager.getColor(SWT.COLOR_RED));
			gc.drawLine((int) gesture[gesture.length-4], (int) gesture[gesture.length-3], 
					(int) gesture[gesture.length-2], (int) gesture[gesture.length-1]);

			gc.setForeground(SWTResourceManager.getColor(SWT.COLOR_DARK_GRAY));
			gc.setLineWidth(1);
		}

		ImageLoader loader = new ImageLoader();
		loader.data = new ImageData[] {image.getImageData().scaledTo(width, height)};
		loader.save(folder + "\\" + name + ".png", SWT.IMAGE_PNG);
	}

	private void centerGesture(int size) {;
	try {
		Vector2 centerPosition =  new Vector2();

		centerPosition.x = ((size / 2) - (bounds.width / 2)) - bounds.x;
		centerPosition.y = ((size / 2) - (bounds.height / 2)) - bounds.y;

		bounds.x += centerPosition.x;
		bounds.y += centerPosition.y;

		for (int i = 0; i < gesture.length; i+=2) {
			gesture[i] += centerPosition.x;
			gesture[i+1] += centerPosition.y;
		}
	} catch (NullPointerException npe) {
		// TODO: handle exception
	}
	}

	public void setGesturing(boolean gesturing) {
		this.gesturing = gesturing;
	}

	public void addStroke(Vector2 vector) {
		strokes.add(vector);
	}

	public void resetStroke() {
		strokes.clear();
		gesturing = true;
	}

	private void configGC(GC gc) {
		gc.setLineWidth(5);
		gc.setAntialias(SWT.ON);
		gc.setAdvanced(true);
		gc.setInterpolation(SWT.HIGH);
	}

	public void setGesture(float[] points) {
		this.gesture = points;
	}

	public float[] getGesture() {
		return gesture;
	}

	public void setLibrary(GestureLibrary library) {
		this.mLibrary = library;
	}

	public Gesture getInput() {
		return mCurrentGesture;
	}

	public void clear() {
		mCurrentGesture = null;
		strokes.clear();
		gesture = new float[1024];
		redraw();
	}

	public void setBounds(Rectangle bounds) {
		this.bounds = bounds;
	}
}
