package tempest.app.manager.composite;

import java.io.File;
import java.io.FileFilter;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Link;

import tempest.app.manager.GameManager;
import tempest.app.manager.GameManager.Layouts;
import tempest.app.manager.factories.ElementsFactory;
import tempest.app.manager.utils.Configuration;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;

public class HomeComposite extends Composite {

	private Group groupGesturesList;
	private Group groupAchievementsList;
	private Group groupProfilesList;
	
	private GameManager owner;

	public HomeComposite(Composite parent, int style, GameManager owner) {
		super(parent, style);
		this.owner = owner;
		
		compositeLayout();
		
		compositeGesturesGroup();
		compositeAchievementsGroup();
		compositeMiscGroup();
	}
	
	public void seedGesturesList() {
		try {
			File folder = new File(Configuration.asString("Project.gestures", ""));
			File[] list = folder.listFiles(new FileFilter() {

				@Override
				public boolean accept(File pathname) {
					return pathname.getName().endsWith(".gestures") || pathname.getName().endsWith(".GESTURES");
				}
			});
			
			for (File file : list) {
				ElementsFactory.createGestureLink(groupGesturesList, file.getName(), owner);
			}

		} catch (NullPointerException e) {
			System.err.println("Gestures property not found in config...");
		}
		groupGesturesList.layout();
	}
	
	public void seedAchievementsList() {
		try {
			ElementsFactory.createAchievementLink(groupAchievementsList, Configuration.asString("Achievements.xml", ""), owner);
		} catch (NullPointerException e) {
			System.err.println("Achievements.xml property not found...");
		}
		
		groupAchievementsList.layout();
	}

	public void compositeMiscGroup() {
		Group grpMisc = new Group(this, SWT.NONE);
		grpMisc.setText("Misc");
		grpMisc.setLayout(new GridLayout(1, false));
		
		Link link_2 = new Link(grpMisc, SWT.NONE);
		link_2.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				owner.switchTo(Layouts.PROPERTIES);
			}
		});
		link_2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		link_2.setText("<a>Edit game properties</a>");
		
		groupProfilesList = new Group(grpMisc, SWT.NONE);
		groupProfilesList.setLayout(new GridLayout(4, false));
		groupProfilesList.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		groupProfilesList.setText("Profiles");
	}

	private void compositeAchievementsGroup() {
		Group grpAchievements = new Group(this, SWT.NONE);
		grpAchievements.setText("Achievements");
		grpAchievements.setLayout(new GridLayout(1, false));
		
		Link link_1 = new Link(grpAchievements, SWT.NONE);
		link_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		link_1.setText("<a>Create new achievement xml</a>");
		
		groupAchievementsList = new Group(grpAchievements, SWT.NONE);
		groupAchievementsList.setLayout(new GridLayout(1, false));
		groupAchievementsList.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		groupAchievementsList.setText("Project achievements");
	}

	private void compositeGesturesGroup() {
		Group grpGestures = new Group(this, SWT.NONE);
		grpGestures.setText("Gestures");
		grpGestures.setLayout(new GridLayout(1, false));
		
		Link link = new Link(grpGestures, SWT.NONE);
		link.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		link.setText("<a>Create new gesture library</a>");
		
		Link link_3 = new Link(grpGestures, SWT.NONE);
		link_3.setText("<a>Browse existing gesture library...</a>");
		
		groupGesturesList = new Group(grpGestures, SWT.NONE);
		groupGesturesList.setLayout(new GridLayout(4, false));
		groupGesturesList.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		groupGesturesList.setText("Project libraries");
	}

	private void compositeLayout() {
		FillLayout fillLayout = new FillLayout(SWT.VERTICAL);
		fillLayout.spacing = 10;
		fillLayout.marginHeight = 10;
		fillLayout.marginWidth = 10;
		setLayout(fillLayout);
	}

	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}
}
