package tempest.app.manager.composite;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;

import tempest.app.manager.factories.PropertiesFactory;
import tempest.app.manager.utils.GameConfiguration;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.custom.ScrolledComposite;

public class PropertiesComposite extends Composite {
	private Text gestures;
	private Button fpsButton;
	private Button debugButton;
	private ScrolledComposite scrolledComposite;
	private Composite composite;
	private Label lblAchievementsmaxpergame;
	private Text achievementPerGame;

	public PropertiesComposite(Composite parent, int style) {
		super(parent, style);
		GameConfiguration.loadInstance();
		addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				try {
					GameConfiguration.saveInstance();
				} catch (NullPointerException ex) {}

			}
		});
		compositeLayout();

		compositeAppGroup();
		compositeWagonGroup();
		compositeEnemyGroup();
	}

	public void updateStats() {
		fpsButton.setSelection(GameConfiguration.asBool("App.fps", true));
		debugButton.setSelection(GameConfiguration.asBool("App.debug", true));

		achievementPerGame.setText(Integer.toString(GameConfiguration.asInt("Achievements.maxpergame", 0)));

		gestures.setText(GameConfiguration.asString("App.gestures", "error"));
	}

	private void compositeEnemyGroup() {
	}

	private void compositeWagonGroup() {
	}

	private void compositeAppGroup() {

		scrolledComposite = new ScrolledComposite(this, SWT.H_SCROLL | SWT.V_SCROLL);
		scrolledComposite.setExpandHorizontal(true);
		scrolledComposite.setExpandVertical(true);

		composite = new Composite(scrolledComposite, SWT.NONE);
		composite.setLayout(new FillLayout(SWT.VERTICAL));
		Group grpApp = new Group(composite, SWT.NONE);
		grpApp.setText("App");
		grpApp.setLayout(new GridLayout(3, false));

		Label lblAppfps = new Label(grpApp, SWT.NONE);
		lblAppfps.setText("App.fps");

		fpsButton = new Button(grpApp, SWT.CHECK);
		fpsButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				GameConfiguration.putProperty("App.fps", Boolean.toString(fpsButton.getSelection()));
			}
		});
		new Label(grpApp, SWT.NONE);

		Label lblAppdebug = new Label(grpApp, SWT.NONE);
		lblAppdebug.setText("App.debug");

		debugButton = new Button(grpApp, SWT.CHECK);
		debugButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				GameConfiguration.putProperty("App.debug", Boolean.toString(debugButton.getSelection()));
			}
		});
		new Label(grpApp, SWT.NONE);

		Label lblAppgestures = new Label(grpApp, SWT.NONE);
		lblAppgestures.setText("App.gestures");

		gestures = new Text(grpApp, SWT.BORDER);
		gestures.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

		Button btnNewButton = new Button(grpApp, SWT.NONE);
		btnNewButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				FileDialog gestures = new FileDialog(getShell(), SWT.OPEN);
				gestures.setFilterExtensions(new String[]{"*.gestures"});
				gestures.setFilterNames(new String[]{"Gestures(*.gestures)"});
			}
		});
		btnNewButton.setText("Browse");

		lblAchievementsmaxpergame = new Label(grpApp, SWT.NONE);
		lblAchievementsmaxpergame.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblAchievementsmaxpergame.setText("Achievements.maxpergame");

		achievementPerGame = new Text(grpApp, SWT.BORDER);
		new Label(grpApp, SWT.NONE);

		PropertiesFactory.create(composite, "Wagon", new PropertiesFactory.Elements[]{
				PropertiesFactory.Elements.HEALTH,
				PropertiesFactory.Elements.VELOCITY
		});

		PropertiesFactory.create(composite, "Enemy", new PropertiesFactory.Elements[] {
				PropertiesFactory.Elements.HEALTH,
				PropertiesFactory.Elements.VELOCITY
		});

		PropertiesFactory.create(composite, "Fireball", new PropertiesFactory.Elements[] {
				PropertiesFactory.Elements.COOLDOWN,
				PropertiesFactory.Elements.VELOCITY
		});
		
		Group abilities = PropertiesFactory.createEmpty(composite, "Abilities");

		PropertiesFactory.addScalableProperty(abilities, "Groundshake", PropertiesFactory.Elements.COOLDOWN);
		PropertiesFactory.addScalableProperty(abilities, "Lightining", PropertiesFactory.Elements.COOLDOWN);
		
		scrolledComposite.setContent(composite);
		scrolledComposite.setMinSize(composite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	}

	private void compositeLayout() {
		FillLayout fillLayout = new FillLayout(SWT.VERTICAL);
		fillLayout.spacing = 10;
		fillLayout.marginHeight = 10;
		fillLayout.marginWidth = 10;
		setLayout(fillLayout);
	}

	@Override
	protected void checkSubclass() {}
}
