package tempest.app.manager.dialogs;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.wb.swt.SWTResourceManager;
import org.eclipse.swt.widgets.Button;

import tempest.app.manager.GameManager;
import tempest.app.manager.utils.Configuration;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;

public class GenerateThumbnails extends Dialog {
	private Text widthText;
	private Text heightText;
	private Text destinationText;

	private String destination;

	private boolean link = true;
	private Label widthLabel;
	private Label heightLabel;

	public GenerateThumbnails(GameManager manager) {
		super(manager.shell);
	}

	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container = compositeContent(parent);

		widthText.setText(Configuration.asString("Gestures.thumbs.width", "32"));
		heightText.setText(Configuration.asString("Gestures.thumbs.height", "32"));

		destinationText.setText(Configuration.asString("Project.assets", "") + "\\gestures\\");
		destinationText.setSelection(destinationText.getText().length());

		return container;
	}

	private Composite compositeContent(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		container.setLayout(new FormLayout());

		Composite composite_1 = new Composite(container, SWT.NONE);
		FormData fd_composite_1 = new FormData();
		fd_composite_1.right = new FormAttachment(100);
		fd_composite_1.top = new FormAttachment(0, 5);
		fd_composite_1.left = new FormAttachment(0, 5);
		composite_1.setLayoutData(fd_composite_1);
		composite_1.setLayout(new GridLayout(5, false));

		Composite composite = new Composite(composite_1, SWT.NONE);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
		composite.setLayout(new GridLayout(2, false));

		Label lblWidth = new Label(composite, SWT.NONE);
		lblWidth.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblWidth.setText("Width:");

		widthText = new Text(composite, SWT.BORDER);
		widthText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				try {
					Integer.parseInt(widthText.getText());
				} catch (NumberFormatException nfe) {}

				if(link) {
					heightText.setText(widthText.getText());
				}
				widthLabel.setText(widthText.getText());
			}
		});

		Label lblHeight = new Label(composite, SWT.NONE);
		lblHeight.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblHeight.setText("Height:");

		heightText = new Text(composite, SWT.BORDER);
		heightText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				heightLabel.setText(heightText.getText());
			}
		});
		heightText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));

		widthLabel = new Label(composite_1, SWT.NONE);
		GridData gd_widthLabel = new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 1);
		gd_widthLabel.widthHint = 24;
		gd_widthLabel.minimumWidth = 32;
		widthLabel.setLayoutData(gd_widthLabel);
		widthLabel.setFont(SWTResourceManager.getFont("Segoe UI", 12, SWT.NORMAL));
		widthLabel.setText("00");

		Label lblX = new Label(composite_1, SWT.NONE);
		lblX.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 1));
		lblX.setFont(SWTResourceManager.getFont("Segoe UI", 12, SWT.BOLD));
		lblX.setText("X");

		heightLabel = new Label(composite_1, SWT.NONE);
		GridData gd_heightLabel = new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 1);
		gd_heightLabel.widthHint = 24;
		gd_heightLabel.minimumWidth = 32;
		heightLabel.setLayoutData(gd_heightLabel);
		heightLabel.setFont(SWTResourceManager.getFont("Segoe UI", 12, SWT.NORMAL));
		heightLabel.setText("00");

		Button linkDimm = new Button(composite_1, SWT.TOGGLE);
		linkDimm.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				link = link == true ? false : true;
			}
		});
		linkDimm.setSelection(true);
		linkDimm.setText("Link dimmensions");

		Composite composite_2 = new Composite(container, SWT.NONE);
		composite_2.setLayout(new GridLayout(3, false));
		FormData fd_composite_2 = new FormData();
		fd_composite_2.top = new FormAttachment(composite_1, 6);
		fd_composite_2.right = new FormAttachment(100);
		fd_composite_2.left = new FormAttachment(composite_1, 0, SWT.LEFT);
		composite_2.setLayoutData(fd_composite_2);

		Label lblDestination = new Label(composite_2, SWT.NONE);
		lblDestination.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblDestination.setText("Destination:");

		destinationText = new Text(composite_2, SWT.BORDER);
		destinationText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

		Button btnBrowse = new Button(composite_2, SWT.NONE);
		btnBrowse.setText("Browse");
		return container;
	}

	/**
	 * Create contents of the button bar.
	 * @param parent
	 */
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
				true);
		createButton(parent, IDialogConstants.CANCEL_ID,
				IDialogConstants.CANCEL_LABEL, false);
	}

	/**
	 * Return the initial size of the dialog.
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(450, 300);
	}

	@Override
	protected void okPressed() {
		destination = destinationText.getText();
		GameManager.gestures.generateThumbs(destination, 
				Integer.parseInt(widthLabel.getText()), Integer.parseInt(widthLabel.getText()));
		System.out.println("Success!");

		this.close();
	}
}
