package tempest.app.manager.factories;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Scale;
import org.eclipse.swt.widgets.Text;

import tempest.app.manager.utils.GameConfiguration;

public class PropertiesFactory {
	public static enum Elements {
		COOLDOWN, VELOCITY, HEALTH;
	}

	public static Group create(final Composite parent, final String title, final Elements[] elements) {
		Group group = new Group(parent, SWT.NONE);
		group.setText(title);
		group.setLayout(new GridLayout(3, false));

		for (Elements e : elements) {
			final String element = e.toString().toLowerCase();
			final String property = title + "." + element.toLowerCase();

			final Label l = new Label(group, SWT.NONE);
			final Scale s = new Scale(group, SWT.NONE);
			final Text t = new Text(group, SWT.BORDER);

			switch (e) {
			case VELOCITY:
				configureVelocity(property, s, t);
				break;
			case HEALTH:
				configureHealth(property, s, t);
				break;
			case COOLDOWN:
				configureCooldown(property, s, t);
				break;
			}
			s.notifyListeners(SWT.Selection, null);
			s.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

			GridData gridData = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);
			gridData.widthHint = 32;
			t.setLayoutData(gridData);

			l.setText(element.substring(0, 1).toUpperCase() + element.substring(1));
		}
		return group;
	}

	public static Group createEmpty(Composite parent, String title) {
		Group group = new Group(parent, SWT.NONE);
		group.setText(title);
		group.setLayout(new GridLayout(3, false));
		return group;
	}

	public static void addScalableProperty(Group group, final String title, final Elements element) {
		final String property = title + "." + element.toString().toLowerCase();

		final Label l = new Label(group, SWT.NONE);
		final Scale s = new Scale(group, SWT.NONE);
		final Text t = new Text(group, SWT.BORDER);

		switch (element) {
		case VELOCITY:
			configureVelocity(property, s, t);
			break;
		case HEALTH:
			configureHealth(property, s, t);
			break;
		case COOLDOWN:
			configureCooldown(property, s, t);
			break;
		}
		s.notifyListeners(SWT.Selection, null);
		s.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

		GridData gridData = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);
		gridData.widthHint = 32;
		t.setLayoutData(gridData);

		l.setText(property);
	}

	private static void configureCooldown(final String property, final Scale s,
			final Text t) {
		s.setMaximum(100);
		s.setSelection((int) (GameConfiguration.asFloat(property, 0) * 10));

		s.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				t.setText(Float.toString(s.getSelection() / 10.0f));
			}
		});

		t.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				try {
					Float.parseFloat(t.getText());
					GameConfiguration.putProperty(property, t.getText());
					s.setSelection((int) (Float.parseFloat(t.getText()) * 10));
				} catch (NumberFormatException ex) {
				}
			}
		});
	}

	private static void configureHealth(final String property, final Scale s,
			final Text t) {
		s.setMaximum(100);
		s.setMinimum(1);
		s.setSelection(GameConfiguration.asInt(property, 0));

		s.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				t.setText(s.getSelection() + "");
			}
		});

		t.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				try {
					Integer.parseInt(t.getText());
					GameConfiguration.putProperty(property, t.getText());
					s.setSelection(Integer.parseInt(t.getText()));
				} catch (NumberFormatException ex) {
				}
			}
		});
	}

	private static void configureVelocity(final String property, final Scale s,
			final Text t) {
		s.setMaximum(420);
		s.setMinimum(1);
		s.setSelection((int) (GameConfiguration.asFloat(property, 0) * 10));

		s.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				t.setText(Float.toString(s.getSelection() / 10.0f));
			}
		});

		t.addModifyListener(new ModifyListener() {	
			@Override
			public void modifyText(ModifyEvent e) {
				try {
					Float.parseFloat(t.getText());
					GameConfiguration.putProperty(property, t.getText());
					s.setSelection((int)(Float.parseFloat(t.getText()) * 10));
				} catch (NumberFormatException ex) {
				}
			}
		});
	}
}
