package tempest.app.manager.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Vector;

public class Configuration {
	private static SortedProperties properties;
	private static String value;
	private static File file;

	public static final boolean loadInstance() {
		boolean loaded = false;
		properties = new SortedProperties();
		file = new File(ManagerConstants.PROGRAM_CONFIG);

		if(!file.exists()) {
			try {
				FileOutputStream fileOut = new FileOutputStream(file);
				properties.store(fileOut, "Tempest Manager Configuaration");
				fileOut.close();
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		try {
			FileInputStream fis = new FileInputStream(file);
			properties.load(fis);
			fis.close();
			loaded = true;
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return loaded;
	}

	public static final void saveInstance() {
		try {
			FileOutputStream fos = new FileOutputStream(ManagerConstants.PROGRAM_CONFIG);
			properties.store(fos, System.getProperty("user.name") + "`s Tempest Manager Configuaration");
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public static void putProperty(String name, String value) {
		properties.setProperty(name, value);
	}

	public static int asInt(String key, int defaultValue) {
		value = properties.getProperty(key);
		if(value == null) return defaultValue;
		return Integer.parseInt(value);
	}

	public static float asFloat(String key, float defaultValue) {
		value = properties.getProperty(key);
		if(value == null) return defaultValue;
		return Float.parseFloat(value);
	}

	public static boolean asBool(String key, boolean defaultValue) {
		value = properties.getProperty(key);
		if(value == null) return defaultValue;
		return Boolean.parseBoolean(value);
	}

	public static String asString(String key, String defaultValue) {
		return properties.getProperty(key, defaultValue);
	}	

	private static class SortedProperties extends Properties {
		private static final long serialVersionUID = 1L;

		@SuppressWarnings({ "unchecked", "rawtypes" })
		public synchronized Enumeration keys() {
			Enumeration keysEnum = super.keys();
			Vector keyList = new Vector();
			while(keysEnum.hasMoreElements()){
				keyList.add(keysEnum.nextElement());
			}
			Collections.sort(keyList);
			return keyList.elements();
		}
	}
}
