package tempest.app.manager.wizard;

import java.io.File;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;

import tempest.app.manager.utils.Configuration;

/** Project root Wizard page (Second page)
 * 
 * @author Asen Lekov <asenlekoff@gmail.com
 * @since 01/21/2013
 * @version 1.3
 */
public class SecondRootSelection extends WizardPage {
	private static Text text;

	public SecondRootSelection() {
		super("wizardPage");
		setTitle("Step 1: Project root");
		setDescription("Select root folder of project.");
		setPageComplete(false);
	}

	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);

		setControl(container);
		container.setLayout(new GridLayout(2, false));

		text = new Text(container, SWT.BORDER);
		text.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

		Button browse = new Button(container, SWT.NONE);
		browse.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				DirectoryDialog openDialog = new DirectoryDialog(getShell());
				openDialog.setFilterPath(text.getText());

				openDialog.setText("Directory selection");
				openDialog.setMessage("Select project directory");

				String directory = openDialog.open();

				if(directory == null)
					return;
				text.setText(directory);
				setPageComplete(true);
				updateConfiguration(directory);
			}

			private void updateConfiguration(String directory) {
				File file = new File(directory);
				if (file.exists() && file.isDirectory()) {
					Configuration.putProperty("Project.root", directory);
				}

				file = new File(directory + "\\" + Configuration.asString("Project.name", ""));
				if (file.exists() && file.isDirectory()) {
					Configuration.putProperty("Project.main", directory + "\\" + 
							Configuration.asString("Project.name", ""));
				}

				file = new File(directory + "\\" + Configuration.asString("Project.name", "") + "-android");
				if (file.exists() && file.isDirectory()) {
					Configuration.putProperty("Project.android", directory + "\\" + 
							Configuration.asString("Project.name", "") + "-android");
				}

				file = new File(directory + "\\" + Configuration.asString("Project.name", "") + "-desktop");
				if (file.exists() && file.isDirectory()) {
					Configuration.putProperty("Project.desktop", directory + "\\" + 
							Configuration.asString("Project.name", "") + "-desktop");
				}

				file = new File(Configuration.asString("Project.android", "") + "\\assets\\achievements\\achievments.xml");
				if (file.exists() && file.isFile()) {
					Configuration.putProperty("Achievements.xml", 
							Configuration.asString("Project.android", "") + "\\assets\\achievements\\achievments.xml");
				}

				file = new File(Configuration.asString("Project.android", "") + "\\assets\\achievements\\schema.xsd");
				if (file.exists() && file.isFile()) {
					Configuration.putProperty("Achievements.schema", 
							Configuration.asString("Project.android", "") + "\\assets\\achievements\\schema.xsd");
				}

				file = new File(Configuration.asString("Project.android", "") + "\\assets\\shinygame.properties");
				if(file.exists() && file.isFile()) {
					Configuration.putProperty("Project.properties", 
							Configuration.asString("Project.android", "") + "\\assets\\shinygame.properties");
				}

				file = new File(Configuration.asString("Project.android", "") + "\\assets");
				if (file.exists() && file.isDirectory()) {
					Configuration.putProperty("Project.assets", 
							Configuration.asString("Project.android", "") + "\\assets");
				}

				file = new File(Configuration.asString("Project.desktop", "") + "\\data");
				if(file.exists() && file.isDirectory()) {
					Configuration.putProperty("Project.data", 
							Configuration.asString("Project.desktop", "") + "\\data");
				}
				
				Configuration.putProperty("Project.gestures", Configuration.asString("Project.assets", "") + "\\gestures\\");
				
				Configuration.putProperty("Gestures.thumbs.width", "64");
				Configuration.putProperty("Gestures.thumbs.height", "64");

				Configuration.saveInstance();
			}
		});
		browse.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		browse.setText("Browse...");
	}
}
