package tempest.app.manager.wizard;

import java.io.File;
import java.io.FileFilter;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.layout.GridData;

import tempest.app.manager.utils.Configuration;
import org.eclipse.wb.swt.SWTResourceManager;

/** Files review Wizard page (final page)
 * 
 * @author Asen Lekov <asenlekoff@gmail.com>
 * @since 01/21/2013
 * @version 1.0
 */
public class ThirdFileReview extends WizardPage {
	private Text mainProjectText;
	private Text androidProjectText;
	private Text desktopProjectText;
	private Text xmlText;
	private Text xmlShchemaText;
	private Text propertiesText;
	private Label gesturesLebel;
	private Label profilesLabel;

	public ThirdFileReview() {
		super("wizardPage");
		setTitle("Step 3: Review project files and folders");
		setDescription("Found listed files below.");
		setPageComplete(false);
	}

	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);

		setControl(container);
		container.setLayout(new GridLayout(2, false));

		Label lblMainProject = new Label(container, SWT.NONE);
		lblMainProject.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblMainProject.setText("Main project:");

		mainProjectText = new Text(container, SWT.BORDER);
		mainProjectText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));


		Label lblAndroidProject = new Label(container, SWT.NONE);
		lblAndroidProject.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblAndroidProject.setText("Android project:");

		androidProjectText = new Text(container, SWT.BORDER);
		androidProjectText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));


		Label lblNewLabel = new Label(container, SWT.NONE);
		lblNewLabel.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblNewLabel.setText("Desktop project:");

		desktopProjectText = new Text(container, SWT.BORDER);
		desktopProjectText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		new Label(container, SWT.NONE);
		new Label(container, SWT.NONE);

		Label lblGestures = new Label(container, SWT.NONE);
		lblGestures.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblGestures.setText("Gestures:");

		gesturesLebel = new Label(container, SWT.NONE);
		gesturesLebel.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		gesturesLebel.setForeground(SWTResourceManager.getColor(SWT.COLOR_RED));
		gesturesLebel.setText("0 libraries found");
		new Label(container, SWT.NONE);
		new Label(container, SWT.NONE);

		Label lblUserProfiles = new Label(container, SWT.NONE);
		lblUserProfiles.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblUserProfiles.setText("User profiles:");

		profilesLabel = new Label(container, SWT.NONE);
		profilesLabel.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		profilesLabel.setForeground(SWTResourceManager.getColor(SWT.COLOR_RED));
		profilesLabel.setText("0 profiles found");
		new Label(container, SWT.NONE);
		new Label(container, SWT.NONE);

		Label lblAchievementsXml = new Label(container, SWT.NONE);
		lblAchievementsXml.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblAchievementsXml.setText("Achievements XML:");

		xmlText = new Text(container, SWT.BORDER);
		xmlText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

		Label lblAchievementsXmlSchema = new Label(container, SWT.NONE);
		lblAchievementsXmlSchema.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblAchievementsXmlSchema.setText("Achievements XML Schema:");

		xmlShchemaText = new Text(container, SWT.BORDER);
		xmlShchemaText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		new Label(container, SWT.NONE);
		new Label(container, SWT.NONE);

		Label lblPropertiesFile = new Label(container, SWT.NONE);
		lblPropertiesFile.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblPropertiesFile.setText("Properties file:");

		propertiesText = new Text(container, SWT.BORDER);
		propertiesText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
	}

	public void updateFields() {
		updateGestures();
		updateProfiles();
		Configuration.saveInstance();

		mainProjectText.setText(Configuration.asString("Project.main", "none"));
		androidProjectText.setText(Configuration.asString("Project.android", "none"));
		desktopProjectText.setText(Configuration.asString("Project.desktop", "none"));
		xmlText.setText(Configuration.asString("Achievements.xml", ""));
		xmlShchemaText.setText(Configuration.asString("Achievements.schema", ""));
		propertiesText.setText(Configuration.asString("Project.properties", ""));
	}

	private void updateProfiles() {
		String label = "0 profiles found";

		File folder = new File(Configuration.asString("Project.data", ""));
		File[] list = folder.listFiles(new FileFilter() {

			@Override
			public boolean accept(File pathname) {
				return pathname.getName().endsWith(".tempest") || pathname.getName().endsWith(".TEMPEST");
			}
		});

		if (list.length == 1) {
			label = (list.length + " profile found:\t" + list[0].getName());
		} else {
			label = (list.length + " profiles found:\t");
			for (File f : list) {
				label += "; " + f.getName();
			}
		}

		Configuration.putProperty("Profiles.count", Integer.toString(list.length));
		profilesLabel.setForeground(SWTResourceManager.getColor(SWT.COLOR_DARK_GREEN));
		profilesLabel.setText(label);
	}

	private void updateGestures() {
		String label = "0 libraries found";

		File folder = new File(Configuration.asString("Project.gestures", ""));
		File[] list = folder.listFiles(new FileFilter() {

			@Override
			public boolean accept(File pathname) {
				return pathname.getName().endsWith(".gestures") || pathname.getName().endsWith(".GESTURES");
			}
		});

		if (list.length == 1) {
			label = (list.length + " library found:\t" + list[0].getName());
		} else {
			label = (list.length + " librarires found:\t");
			for (File f : list) {
				label += "; " + f.getName();
			}
		}

		Configuration.putProperty("Gestures.count", Integer.toString(list.length));
		gesturesLebel.setForeground(SWTResourceManager.getColor(SWT.COLOR_DARK_GREEN));
		gesturesLebel.setText(label);
	}
}
